#!/usr/bin/env python3

# @author Augustin Mortier
# @email augustinm@met.no
# @desc A-Profiles Plotter

import pandas as pd
import xarray as xr
import numpy as np
import matplotlib.pyplot as plt

class Plotter:
    def __init__(self, data):    
        self.data = data

    def plot(self, var, **args):
        #if dataset
        if type(self.data) == xr.core.dataset.Dataset:
            _PlotQLFromDataset(self.data, var, **args).plot()

class _PlotQLFromDataset:
    """[summary]

    Args:
        data (dixarray.core.dataset.Datasetct): dataset to be plotted
    """
    def __init__(self, data, var, vmin=0, vmax=1, cmap='Spectral_r'):
        self.data = data
        self.var = var
        self.vmin = vmin
        self.vmax = vmax
        self.cmap = cmap
    
    def plot(self):
        fig, axs = plt.subplots(1, 1, figsize=(6, 3))
        #plot image
        #plt.imshow(self.data[self.var],origin='lower')
        X = self.data.time
        Y = self.data.altitude
        C = np.transpose(self.data[self.var].values)
        vmin =self.vmin
        vmax = self.vmax
        cmap = self.cmap
        plt.pcolormesh(X, Y, C, vmin=vmin, vmax=vmax, cmap=cmap, shading='nearest')
        
        #set title and axis
        yyyy = pd.to_datetime(self.data.time.values[0]).year
        mm = pd.to_datetime(self.data.time.values[0]).month
        dd = pd.to_datetime(self.data.time.values[0]).day
        station_id = self.data.instrument_id
        plt.title('{} - {}/{:02}/{:02}'.format(station_id, yyyy, mm, dd))
        plt.xlabel('Time')
        plt.ylabel('Altitude (m)')
        
        #colorbar
        cbar = plt.colorbar()
        cbar.set_label('Attenuated backscatter Signal')

        plt.show()

def main():
    import reader
    path = "data/e-profile/2021/09/08/L2_0-20000-006735_A20210908.nc"
    path = "tests/data/L2_0-20000-006735_A20210908.nc"
    apro_reader = reader.ReadProfiles(path)
    l2_data = apro_reader.read()

    #plot the dataset
    plotter = Plotter(l2_data)
    plotter.plot('attenuated_backscatter_0',vmin=0, vmax=2, cmap='viridis')

if __name__ == '__main__':
    main()
    


#!/usr/bin/env python3

# @author Augustin Mortier
# @email augustinm@met.no
# @desc A-Profiles Reader

import urllib.request
import io
import xarray as xr

class ReadProfiles:
    """Class for reading vertical profiles.

    Note:
        Do not include the `self` parameter in the ``Args`` section.

    Attributes:
        path (str): path of path of the file to be read.
    """

    def __init__(self, path):
        self.path = path

    def _get_reader(self):
        """Method which gets the right reader class.

        Note:
            Based on the file name, we return the relevant reading class.

        Returns:
            <class '__main__._ReadL2EPROFILE'>: reader class
        """

        #check if the filename starts with L2
        filename = self.path.split('/')[-1]
        if filename[0:2] == 'L2':
            return _ReadL2EPROFILE
        else:
            raise NotImplementedError('Only L2 files are supported at the moment')

    def read(self):
        """Method which calls the relevant reading class.

        Note:
            Based on the file name, we call the relevant reading class.

        Returns:
            <class 'xarray.Dataset'>: xarray Dataset
        """
        
        #get the right reading class
        reading_class = self._get_reader()
        #calls the read function from the right reading class
        data = reading_class(self.path).read_file()
        return data

class _ReadL2EPROFILE:
    """Read Level 2 data E-PROFILE single NetCDF file.
    
    Note:
        This function can also fetch data from the E-PROFILE hub:
        https://data.ceda.ac.uk/badc/eprofile/data/
        https://data.ceda.ac.uk/badc/eprofile/data/daily_files/

    Attributes:
        path (str): The path of the file to be read (path or URL).

    Returns:
        <class 'xarray.Dataset'>
    """

    def __init__(self, path):
        self.path = path

    def check_path(self):
        """Check the path

        Raises:
            OSError: [description]
            NotImplementedError: [description]
        """
        #check if file is NetCDF
        filename = self.path.split('/')[-1]
        if filename[-3:] != '.nc':
            raise OSError("NetCDF: Unknown file format:{}".format(filename))
        #check if web address
        if self.path[0:5] == 'https':
            raise NotImplementedError("The reading of CEDA Archive is not implemented yet")

    def read_file(self):
        self.check_path()
        dataset = xr.open_dataset(self.path)
        return dataset


def main():
    #path = "https://dap.ceda.ac.uk/badc/eprofile/data/norway/oslo/met-norway-jenoptick-chm15k-nimbus_A/2021/06/26/L2_0-20000-0-01492_A202106260005.nc"
    path = "data/e-profile/2021/09/08/L2_0-20000-006735_A20210908.nc"
    #path = "tests/data/test.txt"
    l2_data = ReadProfiles(path).read()
    print(l2_data)

if __name__ == '__main__':
    main()